#if !defined(__cartanal_h)
#define __cartanal_h

/* $Id: cartanal.h,v 1.1.1.1 1996/03/27 18:04:20 cvs Exp $
 *
 * Copyright (c) 1996 University of Alberta
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See COPYING file for more information.
 */

#define id_MAINMENU 	  	"MAINMENU"
#define id_FILENAMELB	  	901
#define dlg_SELECTLINE 	  	"SELECTLINE"
#define dlg_ABOUT			"ABOUT"
#define DLG_PREFERENCES   	"PREFERENCES"
#define DLG_HANDCOUNT   	"HANDCOUNT"
#define DLG_FILELOCATIONS 	"FILELOCATIONS"
#define icn_MAIN	  		"MAINICON"
#define LByofs		  		50

#define Pi			3.14159265359
#define numSegments	64	// number of sides in circle
#define blocksize 	64
#define nextBlockSize	48
#define edgesize 	8   // size of border within block
#define overShoot	1	// overshoot past minimum intensity
#define maxVar		1	// maximum rise from minimum value
#define numSmooths	2	// number of radii smoothes

#define minCellSizeDefault  5   	// minimum pixels needed per cell default
#define maxPixsDefault		0		// maximum pixels used for color measure (0=infinite)
#define saveEnhancedDefault 0		// saved Enhanced image before counting
#define use24BitDefault		0
#define idealSatDefault		100
#define backgroundStDevsDefault	10
#define stackUsageDefault	0
#define histogramDefault	0
#define RGBValuesDefault	0
#define extraSmoothDefault	0
#define localizationDefault 0		// 0=global, 1=medium, 2=local

#define thresDefault        96	 	// minimum intensity default
#define useOldEnhanceDefault	0

#define StatusInterval 500
#define maxpoints 	150
#define fsPathName	128
#define TGAExtension	".tga"
#define BMPExtension    ".bmp"
#define DIBExtension    ".dib"
#define EnhExtension	".enh"
#define LineExtension	".lin"
#define DataExtension	".dat"
#define HandDataExtension	".hnd"
#define HLPFileName     "cartanal.hlp"
#define INIFileName     "analysis.ini"
#define CartanalSection	"Cartilage Analysis"
#define specialEnhancedType	0x41484E45

#define circleSizeSmall 3
#define circleSizeMedium 6
#define circleSizeLarge 9

struct tline {
  int numpoints;
  TPoint points[maxpoints];
  };

struct selectlinemsg {
  int top, s_i, i_d, deep, left, right;
  };


// classes

typedef TArrayAsVector<TPoint> TCellPoints;
typedef TArrayAsVectorIterator<TPoint> TCellPointsIterator;


class TPreferences
  {
  public:
	TPreferences();
	void RestoreDefaults();
	void LoadPreferences();
	void SavePreferences();
	inline const char* GetLocalIniFileName() { return iniFile; }

	int minCellSize,        // minimum pixels needed per cell
		maxPixs,			// maximum pixels used for color determination (0=infinite)
		defSaveEnhanced,	// saved Enhanced image before counting
		use24Bit,			// use 24-bit DIB for display
		idealSat,			// ideal saturation 10-255
		backgroundStDevs,	// number of StDev's from mean *10 (1-100)
		extraSmooth,
		localization,		// localization of features (0=global, 2=local)
		displayStackUsage,
		displayHistogram,
		displayRGBValues,
		useOldEnhance,		// use version 1.x enhancement algorithm
		thres;       		// minimum intensity

	char DATPath[fsPathName];

  protected:
	char iniFile[fsPathName];
  };

class TPreferencesDialog : public TDialog
  {
  protected:
	TEdit *MinCellSize, *DATPath, *BackStDevs, *ImageSat, *MaxPixs;
	TCheckBox *SaveEnh, *Use24Bit, *UseOldEnhance, *ExtraSmooth,
			  *DisplayHistogram, *DisplayStackUsage, *DisplayRGBValues;
	TRadioButton *Local, *Medium, *Global;
	TPreferences *prefs;

  public:
	TPreferencesDialog(TWindow* parent, TPreferences *Aprefs);

	virtual void SetupWindow();
	void CmOk();
	void CmDefaults();

  DECLARE_RESPONSE_TABLE(TPreferencesDialog);
  };

class TDisplayWindow;

class THandCountDialog : public TDialog
  {
  protected:
	TRadioButton *smallC, *med, *large;
	TStatic *red, *green;

	TDisplayWindow *dw;

  public:
	THandCountDialog(TDisplayWindow* parent);

	void SetupWindow();
	void SetCounts(int redCount, int greenCount);
	void BnCircleSize();
	void CmOk();
	void CmUndo();

  DECLARE_RESPONSE_TABLE(THandCountDialog);
  };

class TApp : public TApplication
  {
  public:
	TApp (char far *name) : TApplication (name) {};
	virtual void InitMainWindow ();

	void CmIndex();
	void CmKeyboard();
	void CmProcedures();
	void CmUsingHelp();
	void CmAbout();

  DECLARE_RESPONSE_TABLE (TApp);
  };

class TMainWindow : public TFrameWindow
  {
  protected:
	TStatic _FAR *FuncText;
	TStatic _FAR *StatText;
	TDisplayWindow *DisplayWindow;
	TOpenSaveDialog::TData *FileData;
	TListBox *fileNameLB;

	TStatic *StackText;
	int stackcount, stackmax;
	char stackstr[40];

	int busy, wantToQuit, wantToStop, doingBatch, saveEnhanced;
	char currentFile[fsPathName];
	MemoryImage *image;
	int imageType;  // 0-no image, 1-original, 2-enhanced, 3-counted
	int W, H;
	int numCells;
	ofstream *fout;
	TPreferences prefs;

	// stuff for when maxPixs>0
	int startx, starty, radius, numcolor;

  public:
	TMainWindow (TWindow * AParent);

	void MouseMovePosition(TPoint &point);
	void DoneHandCounting(TCellPoints *cellPoints);
	inline void SetCount(int count) { numCells = count; }

  protected:
	~TMainWindow ();
	int CanClose ();
	void SetupWindow ();

	void CMOpen();
	void CMAddFile();
	void CMRemoveFile();
	void CMPreferences();
	void CMQuit();
	void CMStartBatch();
	void CMStopBatch();
	void CMSaveEnhanced();
	void CMEnhanceImage();
	void CMOldEnhanceImage();
	void CMHandCount();
	void CMCountCells();
	void CMCompressImage();

	int OpenImageFile(char *filename);
	virtual void SetStatus(char *function, int percent=-1);
	virtual void DisplayImage ();

	void FindMax (short int x1, short int y1, short int x2, short int y2);
	void FindObject (int x, int y);
	int FindEdge (int x, int y, int cx, int cy);
	void Line (int x1, int y1, int x2, int y2);
	long DoLine (int x, int y, long &tr, long &tg, long &sx, long &sy);
	long Fill (int &x, int &y, long &tr, long &tg);

	void EvSize (UINT sizeType, TSize &size);
	void EvTimer(UINT timerId);
  DECLARE_RESPONSE_TABLE( TMainWindow );
  };

class TDisplayWindow : public TFrameWindow
  {
  protected:
	TDC *dragDC;
	TPalette *pal;
	TDib *dib;

	int displaying;
	int w, h;
	tline topline, rightline, leftline, boneline, s_iline, i_dline;
	tline curline;
	char lineSaveName[fsPathName];

	THandCountDialog *handCount;
	TCellPoints *cellPoints;
	int redCount, greenCount;
	int circleSize;

  public:
	TDisplayWindow (TWindow * AParent);
	virtual ~TDisplayWindow ();

	virtual void Circle (int x, int y);
	virtual void DisplayImage (char *name, MemoryImage *image);
	virtual void SetLineSaveName (char *name);

	void StartHandCount();
	void DoneHandCount();
	void UndoLastCell();
	inline void SetCircleSize(int size)
	  { circleSize = size;  Invalidate(FALSE); }
	inline int GetCircleSize()
	  { return circleSize; }

  protected:
	virtual void MakePalette();

	virtual int CanClose ();
	virtual void Paint (TDC &PaintDC, BOOL, TRect &);
	virtual void SaveLines ();
	virtual void ClearLines ();

	void EvRButtonDown( UINT, TPoint& );
	void EvLButtonDown( UINT, TPoint& );
	void EvMouseMove( UINT, TPoint& );
	void EvLButtonUp( UINT, TPoint& );
  DECLARE_RESPONSE_TABLE( TDisplayWindow );
  };


class THistogramWindow : public TFrameWindow
  {
  public:
	THistogramWindow (TWindow *AParent, char *title, long *data, int Amean=-1, int Astdev=-1);

  protected:
	long hData[256];
	long max;
	int mean, stdev;

	void Paint (TDC &PaintDC, BOOL, TRect &);
	void EvSize(uint sizeType, TSize& size);

  DECLARE_RESPONSE_TABLE( THistogramWindow );
  };


#endif
