/* $Id: imagedos.cpp,v 1.1.1.1 1994/12/27 17:59:10 cvs Exp $
 *
 * Copyright (c) 1996 University of Alberta
 *
 * May be copied or modified under the terms of the GNU General Public
 * License.  See COPYING file for more information.
 */

#include <image.h>
#include <alloc.h>
#include <graphics.h>

void MemoryImage::MemoryImage (int rBits, int gBits, int bBits)
  {
  imageType = IT_TRUECOLOR;
  redBits = rBits;
  greenBits = gBits;
  blueBits = bBits;
  bitsPerPixel = rBits+gBits+bBits;
  if (bitsPerPixel <= 24)
    while (!bitsPerPixel == (1, 2, 4, 8, 16, 24)) bitsPerPixel++;
  image = 0;  imageHandle = 0;
  histogram = 0;
  StatusFunc = 0;  lastlinenum = -1;
  };

void MemoryImage::MemoryImage (int grayBits)
  {
  imageType = IT_GRAYSCALE;
  bitsPerPixel = grayBits;
  if (bitsPerPixel <= 24)
    while (!bitsPerPixel == (1, 2, 4, 8, 16, 24)) bitsPerPixel++;
  redBits = greenBits = blueBits = 0;
  image = 0;  imageHandle = 0;
  histogram = 0;
  StatusFunc = 0;  lastlinenum = -1;
  };

void MemoryImage::~MemoryImage ()
  {
  if (image) farfree (image);
  if (histogram) delete histogram;
  };

int MemoryImage::CreateImage (int width, int height)
  {
  if (image) farfree (image);
  lastlinenum = -1;
  h = height;
  w = width;
  long size = (w * bitsPerPixel);
  size = ((h * size) + 7) >> 3;
  image = (unsigned char far*) farmalloc (size);
  if (!image) return (-1);
    else return (0);
  };

void MemoryImage::DisplayImage (int xOfs, int yOfs)
  {
  int x, y, r, g, b;

  switch (imageType) {
    case (IT_TRUECOLOR) :
      for (y=0; y<h; y++)
	{
	if (StatusFunc) StatusFunc ((long)100*y/h);
	for (x=0; x<w; x++)
	  {
	  GetPixelTC (x, y, r, g, b);
	  putpixel (x+xOfs, y+yOfs, (g&0xF0) + (r>>4));
	  };
	};
      break;
    case (IT_GRAYSCALE) :
      for (y=0; y<h; y++)
	{
	if (StatusFunc) StatusFunc ((long)100*y/h);
	for (x=0; x<w; x++)
	  {
	  GetPixelGray (x, y, g);
	  putpixel (x+xOfs, y+yOfs, g);
	  };
	};
    };
  };

#pragma argsused
unsigned int MemoryImage::CreateBitmap (unsigned int memDC)
  {
  };
